from lists.list_manager import ListManager
from os.path import (
    join as os_path_join,
    exists as os_path_exists,
    expanduser as os_path_expanduser,
)
from os import remove as os_remove
from json import load as json_load, dump as json_dump


class TestListManager:
    def setup_class(self):
        self.file_name = os_path_join(os_path_expanduser("~"), ".list_manager")
        if os_path_exists(self.file_name):
            self.requires_cleanup = True
            with open(self.file_name, "r") as json_input_file:
                self.stored_lists = json_load(json_input_file)
        else:
            self.requires_cleanup = False

    def teardown_class(self):
        if self.requires_cleanup is True:
            with open(self.file_name, "w") as json_output_file:
                json_dump(self.stored_lists, json_output_file)

    def setup_method(self):
        if os_path_exists(self.file_name):
            os_remove(self.file_name)
        self.list_manager = ListManager()
        self.initial_lists = {"list_a": ["item_a", "item_b"], "list_b": "item_c"}
        self.list_manager.lists = self.initial_lists
        self.list_manager.export_list_to_file()

    def test_init_list_manager_without_file(self):
        assert self.list_manager.lists == self.initial_lists

    def test_init_list_manager_with_file(self):
        new_list_manager = ListManager()
        assert new_list_manager.lists == self.list_manager.lists

    def test_get_lists(self):
        assert self.list_manager.get_lists() == ["list_a", "list_b"]

    def test_get_list_where_list_exists(self):
        assert self.list_manager.get_list("list_a") == self.initial_lists.get("list_a")

    def test_get_list_where_list_does_not_exist(self):
        assert self.list_manager.get_list("list_c") is None

    def test_create_list_where_list_does_not_exist(self):
        assert self.list_manager.create_list(list_name="list_c") is True
        assert "list_c" in self.list_manager.lists

    def test_create_list_where_list_does_not_exist_with_initial_item(self):
        assert self.list_manager.create_list(list_name="list_c", item="item_d") is True
        assert "list_c" in self.list_manager.lists
        assert "item_d" in self.list_manager.lists.get("list_c")

    def test_create_list_where_list_exists(self):
        assert self.list_manager.create_list(list_name="list_a") is False

    def test_delete_list_where_list_exists(self):
        assert self.list_manager.delete_list(list_name="list_a") is True
        assert "list_a" not in self.list_manager.lists

    def test_delete_list_where_list_does_not_exist(self):
        assert self.list_manager.delete_list(list_name="list_c") is False
        assert self.list_manager.lists == self.initial_lists

    def test_add_item_where_list_exists(self):
        assert self.list_manager.add_to_list(list_name="list_a", item="item_d") is True
        assert "item_d" in self.list_manager.lists.get("list_a")

    def test_add_item_where_list_does_not_exist(self):
        assert self.list_manager.add_to_list(list_name="list_c", item="item_d") is False
        assert "list_c" not in self.list_manager.lists

    def test_delete_item_where_list_exists(self):
        assert self.list_manager.delete_from_list(list_name="list_a", item="item_a") is True
        assert "item_a" not in self.list_manager.lists.get("list_a")

    def test_delete_item_where_list_does_not_exist(self):
        assert self.list_manager.delete_from_list(list_name="list_c", item="item_a") is False
        assert self.list_manager.lists == self.initial_lists

    def test_delete_item_where_list_exists_but_item_does_not(self):
        assert self.list_manager.delete_from_list(list_name="list_a", item="item_d") is False
        assert self.list_manager.lists == self.initial_lists



__VERSION__ = "1.0.0"

from lists.list_manager import ListManager
import argparse


def parse_args():  # pragma: no cover
    parser = argparse.ArgumentParser(description="")
    subparsers = parser.add_subparsers(dest="action")

    get_lists = subparsers.add_parser(
        "get_lists", help="Returns All List Names"
    )

    add_item = subparsers.add_parser("add_item", help="Add Item To A List")
    add_item.add_argument(
        "-l",
        "--list",
        help="The List That The Item Is Being Added To",
        action="store",
        type=str,
        required=True,
    )
    add_item.add_argument(
        "-i",
        "--item",
        help="The Item Being Added To The List",
        action="store",
        type=str,
        required=True,
    )

    get_list = subparsers.add_parser(
        "get_list", help="Returns A List And All Included Items",
    )
    get_list.add_argument(
        "-l",
        "--list",
        help="The Name Of The List Being Requested",
        action="store",
        type=str,
        required=True,
    )

    create_list = subparsers.add_parser("create_list", help="Create New List",)
    create_list.add_argument(
        "-l",
        "--list",
        help="The Name Of The List Being Created",
        action="store",
        type=str,
        required=True,
    )
    create_list.add_argument(
        "-i",
        "--item",
        help="An Item To Instantiate The List",
        action="store",
        type=str,
    )

    delete_item_from_list = subparsers.add_parser(
        "delete_item", help="Delete Item From List",
    )
    delete_item_from_list.add_argument(
        "-l",
        "--list",
        help="The Name Of The List Where The Item Is Being Deleted",
        action="store",
        type=str,
        required=True,
    )
    delete_item_from_list.add_argument(
        "-i",
        "--item",
        help="The Name Of The Item Being Deleted",
        action="store",
        type=str,
        required=True,
    )

    delete_list = subparsers.add_parser("delete_list", help="Delete List",)
    delete_list.add_argument(
        "-l",
        "--list",
        help="The Name Of The List Being Deleted",
        action="store",
        type=str,
        required=True,
    )

    return parser.parse_args()


def get_lists(args):  # pragma: no cover
    lists_manager = ListManager()
    lists = lists_manager.get_lists()
    print("Lists:")
    for list_name in lists:
        print(list_name)
    print("\n")


def get_list(args):  # pragma: no cover
    lists_manager = ListManager()
    list_name = lists_manager.get_list(args.list)
    if list_name is not None:
        print(args.list)
        for item in list_name:
            print(item)
    else:
        print(f"{args.list} Does Not Exist")
    print("\n")


def create_list(args):  # pragma: no cover
    lists_manager = ListManager()
    create_list_response = lists_manager.create_list(
        list_name=args.list, item=args.item
    )
    if create_list_response is True:
        print(f"{args.list} Was Successfully Created")
    else:
        print(f"{args.list} Already Exists")
    print("\n")


def add_item(args):  # pragma: no cover
    lists_manager = ListManager()
    add_item_response = lists_manager.add_to_list(
        list_name=args.list, item=args.item
    )
    if add_item_response is True:
        print(f"{args.item} Was Successfully Added To {args.list}")
    else:
        print(f"{args.list} Does Not Exist")
    print("\n")


def delete_item(args):  # pragma: no cover
    lists_manager = ListManager()
    delete_item_response = lists_manager.delete_from_list(
        list_name=args.list, item=args.item
    )
    if delete_item_response is True:
        print(f"{args.item} Was Successfully Deleted From {args.list}")
    else:
        print(
            f"{args.item} Could Not Be Deleted. Please Check That {args.list} "
            f"and {args.item} Exists In It"
        )


def delete_list(args):  # pragma: no cover
    lists_manager = ListManager()
    delete_list_response = lists_manager.delete_list(list_name=args.list)
    if delete_list_response is True:
        print(f"{args.list} Was Successfully Deleted")
    else:
        print(f"{args.list} Does Not Exists")


function_to_function = {
    "get_lists": get_lists,
    "get_list": get_list,
    "create_list": create_list,
    "add_item": add_item,
    "delete_item": delete_item,
    "delete_list": delete_list,
}


def main():  # pragma: no cover
    args = parse_args()
    function_to_function[args.action](args)



from os.path import (
    join as os_path_join,
    exists as os_path_exists,
    expanduser as os_path_expanduser,
)
from json import load as json_load, dump as json_dump


class ListManager:
    def __init__(self):
        self.file_name = os_path_join(os_path_expanduser("~"), ".list_manager")
        if not os_path_exists(self.file_name):
            self.lists = {}
        else:
            with open(self.file_name, "r") as json_input_file:
                json_output = json_load(json_input_file)
                self.lists = json_output.get("lists")

    def get_lists(self):
        return [list_name for list_name in self.lists]

    def get_list(self, list_name):
        if list_name in self.lists:
            return self.lists.get(list_name)
        else:
            return None

    def create_list(self, list_name, item=None):
        if list_name in self.lists:
            return False
        else:
            if item is not None:
                self.lists[list_name] = [item]
            else:
                self.lists[list_name] = []
        self.export_list_to_file()
        return True

    def delete_list(self, list_name):
        if list_name in self.lists:
            self.lists.pop(list_name)
        else:
            return False
        self.export_list_to_file()
        return True

    def add_to_list(self, list_name, item):
        if list_name in self.lists:
            if item not in self.lists.get(list_name):
                self.lists[list_name].append(item)
        else:
            return False
        self.export_list_to_file()
        return True

    def delete_from_list(self, list_name, item):
        if list_name in self.lists:
            if item in self.lists.get(list_name):
                self.lists.get(list_name).remove(item)
            else:
                return False
        else:
            return False
        self.export_list_to_file()
        return True

    def export_list_to_file(self):
        json_output = {"lists": self.lists}
        with open(self.file_name, "w") as json_output_file:
            json_dump(json_output, json_output_file)



from lists import __VERSION__
import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="list-manager",
    version=__VERSION__,
    author="Raghav Nair",
    author_email="nairraghav@hotmail.com",
    description="A simple CLI tool that allows management of lists",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/nairraghav/lists-library",  #
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
    entry_points={
        'console_scripts': ['list_manager=lists:main'],
    }
)
